<?php

/**
 * @file course_relationships.course.inc
 * Holds course module specific hooks.
 */

use Drupal\course\Entity\Course;
use Drupal\course\Entity\CourseEnrollment;


/**
 * Course settings handler callback.
 */
function course_relationships_settings_form() {
  $form = [];

  $units = [
    '86400' => ['max' => 30, 'step size' => 1],
    '3600' => ['max' => 24, 'step size' => 1],
    '60' => ['max' => 60, 'step size' => 1],
  ];

  $form['course_relationships_scan_window'] = [
    '#title' => t('Scan window'),
    '#description' => t('The duration which course relationships will scan regulary for auto-enrollable courses. This is based off of the enrollment creation date.'),
    '#default_value' => variable_get('course_relationships_scan_window', 86400 * 7),
    '#type' => 'timeperiod_select',
    '#units' => $units,
  ];

  $form['course_relationships_cron_chance'] = [
    '#title' => t('Scheduler chance'),
    '#description' => t('Course relationships must scan all previous enrollments and check if the user has access to any auto-enrollable courses. This is the percentage of crons in which this expensive operation will run.'),
    '#default_value' => variable_get('course_relationships_cron_chance', 1),
    '#type' => 'textfield',
    '#size' => 4,
  ];

  $form['course_relationships_enrollment_queue'] = [
    '#title' => t('Enrollment queue limit'),
    '#description' => t('If there is a large backlog of missing enrollments, this number may need to be adjusted. Example: if 500 courses are pending enrollment, this may have to be raised to 1000.'),
    '#default_value' => variable_get('course_relationships_enrollment_queue', 500),
    '#type' => 'textfield',
    '#size' => 8,
  ];

  return system_settings_form($form);
}

/**
 * Implementation of hook_course_enrollment_alter().
 *
 * Apply the parent's enrollment end to the child.
 */
function course_relationships_course_enrollment_presave(CourseEnrollment $enrollment) {
  $node = $enrollment->getCourse();
  if ($node->get('outline')->value == 'relationships') {
    // We need to manually set the timestamp, because a user will never 'take'
    // this course.
    if (empty($enrollment->timestamp)) {
      $enrollment->timestamp = REQUEST_TIME;
    }
  }

  // Apply parent enrollment end date to the child.
  $child_node = $enrollment->getCourse();
  if ($co = course_get_course_object('relationships', 'course', $child_node)) {
    // This course is in a parent.
    $child_node->cr_parent = $co->getCourseNid();
    if ($parent_node = node_load($child_node->cr_parent)) {
      $parent_enrollment = course_enrollment_load($child_node->cr_parent, $enrollment->uid);
      if (!empty($parent_enrollment->enroll_end) && empty($child_node->course['duration'])) {
        // User has a parent enrollment with an end date.
        $enrollment->enroll_end = $parent_enrollment->enroll_end;
      }
      else {
        if (!empty($parent_node->course['duration'])) {
          // User is not enrolled in parent (so end date not set yet), but parent
          // has a duration (enrolling from child).
          // @see course_relationships_course_enroll which runs after this and would
          // set the parent enrollment end. ie the two blocks here should have the
          // same effect.
          $enrollment->enroll_end = time() + $parent_node->course['duration'];
        }
      }
    }
  }
}

/**
 * Implementation of hook_course_enrollment_update().
 */
function course_relationships_course_enrollment_update(CourseEnrollment $enrollment) {
  // Also fire the insert hooks.
  course_relationships_course_enrollment_insert($enrollment);

  $parent_node = $enrollment->getCourse();
  if (!empty($enrollment->enroll_end)) {
    // Loop over all the children and set their enrollment end dates.
    foreach ($parent_node->getObjects() as $courseObject) {
      // Trigger a save on all the children.
      if ($courseObject->getComponent() == 'course') {
        if ($child_enrollment = \Drupal\course\Entity\Course::load($courseObject->getInstanceId())
          ->getEnrollment($enrollment->getUser())) {
          $child_enrollment->save();
        }
      }
    }
  }
}


/**
 * Implements hook_course_enrollment_insert().
 *
 * Enroll user in child courses when enrolled in parent. This will only enroll
 * the user in the child when no other enrollment access hooks fail. Take course
 * access hooks will not prevent the enrollment (i.e., user may not take course
 * because profile is not filled out, but can enroll).
 */
function course_relationships_course_enrollment_insert(CourseEnrollment $enrollment) {
  $course = $enrollment->getCourse();
  $account = $enrollment->getUser();
  $status = $enrollment->status;

  if ($course->get('outline')->value == 'relationships') {
    if ($status && $course->get('parent_auto_enroll')->value == 'auto') {
      // If this is a full enrollment and the course also has auto-enroll
      // enabled, enroll the user in all direct children as long as there are
      // no other blockers.
      foreach ($course->getObjects() as $courseObject) {
        if ($courseObject->bundle() == 'course') {
          /* @var \Drupal\course_relationships\Plugin\course\CourseObject\CourseObjectCourse $courseObject */
          /* @var Course $child_course */
          $child_course = $courseObject->getInstanceEntity();

          $blockers = $child_course->accessMultiple('enroll', $account);
          $block = FALSE;
          foreach ($blockers as $blocktype => $blocker) {
            if (!in_array($blocktype, [
                'course_relationships',
                'course_must_purchase',
              ]) && !$blocker['success']) {
              $block = TRUE;
              break;
            }
          }

          if (!$block) {
            // No enrollment blockers, other than course relationship purchase
            // options.
            if (!$child_course->getEnrollment($account)) {
              // New enrollment.
              $child_course->enroll($account, ['enrollmenttype' => 'course_relationships_descending']);
            }
          }
        }
      }
    }
  }

  if ($courseObject = course_get_course_object('course_relationships', 'course', $course)) {
    // Course is part of another course.
    $course = $courseObject->getCourse();
    if (!course_enrollment_load($courseObject->getCourse()
      ->getNode()->nid, $account->uid)) {
      // User is enrolling in child, without being enrolled in parent. Enroll
      // user in parent.
      // Say we are enrolling from the child of an activity set.
      // Enroll with status 0, we don't want enrollments to descend.
      course_enroll($course->getNode(), $account, 'course_relationships_child', NULL, 0);
    }

    // Check access on the course object representing this course.
    if ($courseObject->access('take', $account)) {
      // Grant access to the course object.
      $courseObject->getFulfillment($account)->grant();
    }

    $course->getEnrollment($account)->track();
  }
}

/**
 * Implements hook_course_enrollment_complete().
 */
function course_relationships_course_enrollment_complete(CourseEnrollment $course_enrollment) {
  $account = $course_enrollment->getUser();
  if ($courseObject = course_get_course_object('course', $course_enrollment->getCourse()->id())) {
    // Complete the course object this course is linked to.
    $courseObject->getFulfillment($account)->setComplete()->save();
  }
}

/**
 * Implements hook_course_enrollment_delete().
 *
 * Descending un-enrollments to children.
 */
function course_relationships_course_enrollment_delete(CourseEnrollment $enrollment) {
  $course = $enrollment->getCourse();

  if (!$account = $enrollment->getUser()) {
    // The account was deleted, so all enrollments are already deleted.
    return;
  }

  foreach ($course->getObjects() as $courseObject) {
    // If parent course has children, un-enroll user from all of them.
    if ($courseObject->getComponent() == 'course') {
      $child = Course::load($courseObject->getInstanceId());
      $child->unenroll($account);
    }
  }

  return;
  // @todo broken
  if ($p_courseObject = course_get_course_object('course_relationships', 'course', $node->id())) {
    $parent = $p_courseObject->getCourse()->getNode();
    $p_enrollment = course_enrollment_load($parent->id(), $user->id());
    if (!$p_enrollment->status) {
      $nids = [];
      foreach ($p_courseObject->getCourse()->getObjects() as $courseObject) {
        if ($courseObject->getOption('module') == 'course_relationships') {
          $nids[] = $courseObject->getInstanceId();
        }
      }
      if (!db_query("SELECT 1 FROM {course_enrollment} ce WHERE nid IN (:nids) AND uid = :uid", [
        ':nids' => $nids,
        ':uid' => $user->id(),
      ])->fetchField()) {
        // User is still enrolled in the parent, has no more child
        // enrollments, but has a status of 0 (only purchased/enrolled in
        // children). Un-enroll them from the parent.
        course_unenroll($p_courseObject->getCourse()->getNode(), $user);
      }
    }
  }
}
