<?php

namespace Drupal\course_relationships\Plugin\Form;

use Drupal\Core\Access\AccessResultAllowed;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\course\Entity\Course;
use Drupal\course\Entity\CourseEnrollment;
use Drupal\field\Entity\FieldConfig;

class CourseRelationshipsBuildForm extends FormBase {

  public function getFormId() {
    return 'course_relationships_build_form';
  }

  /**
   * Linked form wrapper. Calls the recursive form building function that will
   * loop over activity set children.
   *
   * @param array $form_state
   *   Form state passed by drupal_get_form().
   * @param int $node
   *   Parent node object.
   * @param array $children
   *   Node IDs belonging to the parent.
   */
  public function buildForm(array $form, \Drupal\Core\Form\FormStateInterface $form_state) {
    /** @var $node Course */
    $node = $form_state->getBuildInfo()['args'][0];
    global $show_enroll_btn;
    $show_enroll_btn = FALSE;
    $form = $this->buildChildForm($node);

    $form['#node'] = $node;

    $result = $node->access('enroll');

    if ($node->get('parent_enroll_options')->value == 'parent' && !$result) {
      $form['submit']['#access'] = FALSE;
    }
    elseif ($show_enroll_btn) {
      $user = \Drupal::currentUser();
      if ($user->isAuthenticated() && !$node->isEnrolled($user)) {
        $instances = \Drupal::service('entity_field.manager')
          ->getFieldDefinitions('course_enrollment', $node->get('enrollment_type')->value);
        foreach ($instances as $field_name => $field) {
          if (is_a($field, FieldConfig::class)) {
            /* @var $field Drupal\field\Entity\FieldConfig */
            if ($field->getThirdPartySetting('course', 'show_field')) {
              // At least one field must be shown when enrolling. Display the user
              // enrollment form.

                // Create a new enrollment, show them the form.
                $enrollment = CourseEnrollment::create([
                  'cid' => $course->id(),
                  'uid' => $user->id(),
                  'type' => $course->get('enrollment_type')->getString(),
                  'timestamp' => \Drupal::time()->getRequestTime(),
                  'enrollmenttype' => 'self',
                ]);

              $form = \Drupal::service('entity.form_builder')->getForm($enrollment, 'default', ['redirect' => $redirect_url]);

              $form['course_enrollment'] = $form;

              return $form;
            }
          }
        }


      }

      // Show our enrollment button when the user has the option of not only
      // purchasing the parent.
      $form['submit'] = [
        '#type' => 'submit',
        '#value' => t('Enroll'),
        '#access' => $user->isAuthenticated(),
      ];
    }

    // @todo D8 attach the JS
    //drupal_add_js(drupal_get_path('module', 'course_relationships') . '/course_relationships.js');

    //$form['#theme'] = 'course_relationships_build_form';

    return $form;
  }

  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();

    $ret = $this->course_relationships_recursive_add_to_cart($values);

    if (count($ret) == 0) {
      \Drupal::messenger()->addError(t('Please select the items you want to register for.'));
    }

    $all_failed = count(array_filter($ret)) == 0;

    if ($all_failed || (count($ret) && count(array_keys($ret, 'enroll')) == count($ret))) {
      $form_state->setRedirectUrl($form_state->getBuildInfo()['args'][0]->toUrl());
      return;
    }

    if (count($ret) > 0) {
      \Drupal::messenger()->addError('would have redirected to cart');
      //drupal_goto('cart');
    }
  }


  /**
   * Build FAPI form from child NIDs.
   */
  function buildChildForm(Course $parent_node, $level = 0) {
    $user = \Drupal::currentUser();
    global $show_enroll_btn;
    $children = [
      '#tree' => TRUE,
    ];

    $course = $parent_node;
    $nids = [];
    foreach ($course->getObjects() as $courseObject) {
      if ($courseObject->getComponent() == 'course') {
        $nids[] = $courseObject->getInstanceId();
      }
    }
    $nids = array_filter($nids);

    // Construct parent checkbox - disable if appropiate.

    $parent['#title'] = Link::fromTextAndUrl($parent_node->get('title')->value, $parent_node->getUrl())->toString();
    if ((!empty($parent_node->sell_price) && $parent_node->sell_price > 0) && $parent_node->get('parent_enroll_options')->value != 'children') {
      // Activity set is for sale, and setup to allow purchase of this course.
      $parent['#title'] .= '<span class="price child-price"> - ' . uc_currency_format($parent_node->sell_price) . '</span>';
    }
    if ($parent_node->access('update')) {
      // @todo fix for D8
      //$parent['#title'] .= ' ' . Link::fromTextAndUrl('[edit]', $parent_node->getUrl())->toString();
    }
    $parent['#type'] = 'checkbox';

    if (!$user->isAuthenticated()) {
      $parent['#type'] = 'markup';
      $parent['#markup'] = $parent['#title'];
    }

    // @todo need to get all access issues
    $results = $parent_node->accessMultiple('enroll', $user);
    if (!empty($results['course_must_purchase'])) {
      unset($results['course_must_purchase']);
    }
    if (!empty($results)) {
      $result = reset($results);
    }
    else {
      $result = AccessResultAllowed::allowed();
    }

    if ($result->isForbidden()) {
      $parent['#disabled'] = TRUE;
    }

    if ($parent_node->get('parent_enroll_options')->value == 'parent' && $level == 0) {
      // Parent only should disable the topmost checkbox. Check result before
      // prefilling.
      $parent['#default_value'] = $result->isAllowed() ? $parent_node->id() : 0;
      $parent['#disabled'] = TRUE;
    }

    if ($parent_node->get('parent_enroll_options')->value != 'children') {
      $parent['#return_value'] = $parent_node->id();
      if (!empty($parent['#disabled'])) {
        $parent['#value'] = $parent_node->id();
      }

      if ($type = course_relationships_user_has_product($parent_node->id())) {
        // Course enrolled or in cart, disable and check by default.
        $parent['#disabled'] = TRUE;
        $parent['#default_value'] = TRUE;
        $parent['#value'] = $parent_node->id();
        $record = $parent_node->getEnrollment($user);
        // @todo Fix for D9
        //$parent['#description'] = $type == 'Cart' ? '(' . l('in cart', 'cart') . ')' : "($record->section_name)";
      }
    }
    else {
      $parent['#type'] = 'markup';
      $parent['#markup'] = '<div class="form-item">' . $parent['#title'] . '</div>';
    }

    $parent_blockers = $parent_node->accessMultiple('enroll');
    $parent_block = FALSE;
    if (!empty($parent_blockers)) {
      foreach ($parent_blockers as $blocktype => $parent_blocker) {
        if (!in_array($blocktype, [
            'course_must_purchase',
            'course_noanon',
          ]) && $parent_blocker->isForbidden()) {
          $parent_block = TRUE;
          break;
        }
        if (!$parent_node->isEnrolled($user) && !course_relationships_user_has_product($parent_node->id()) && ($parent_blocker->isAllowed() || ($blocktype == 'course_must_purchase' && count($parent_blockers) < 2))) {
          // course has not already been purchased/enrolled in and is not in their cart
          // and is either enrollable or the only blocker is needing to purchase it
          $show_enroll_btn = TRUE;
        }
      }
    }
    elseif (!$parent_node->isEnrolled($user)) {
      // course has not already been enrolled in
      $show_enroll_btn = TRUE;
    }

    if ($parent_block) {

      $image =   [
        '#theme' => 'image',
        '#uri' => 'misc/watchdog-error.png',
        '#title' => $parent['#title'],
      ];
      $image_out = render($image);

      $parent['#title'] = "$image_out {$parent['#title']}<br/>{$parent_blocker->getReason()}";
    }

    $children['parent'] = $parent;

    $children['activities']['#type'] = 'details';
    $children['activities']['#open'] = TRUE;

    $nodes = Course::loadMultiple($nids);
    foreach ($nodes as $node) {

      if ($node->isEnrolled($user)) {
        // User is enrolled so check the take course blockers.
        $blockers = $node->accessMultiple('take', $user);
      }
      else {
        // User not enrolled yet.
        $blockers = $node->accessMultiple('enroll', $user);
      }
      $block = FALSE;
      if (!empty($blockers)) {
        foreach ($blockers as $blocktype => $blocker) {
          if (!in_array($blocktype, [
              'course_must_purchase',
              'course_noanon',
            ]) && $blocker->isForbidden()) {
            $block = TRUE;
            break;
          }
          if (!course_relationships_user_has_product($node->id()) && ($blocker->isAllowed() || ($blocktype == 'course_must_purchase' && count($blockers) < 2))) {
            // course has not already been purchased/enrolled in and is not in their cart
            // and is either enrollable or the only blocker is needing to purchase it
            $show_enroll_btn = TRUE;
          }
        }
      }
      elseif (!$node->getEnrollment($user)) {
        // course has not already been enrolled in
        $show_enroll_btn = TRUE;
      }

      $title = $node->toLink()->toString();

      $child = [];
      $child['#title'] = $title;
      if ((!empty($node->sell_price) && $node->sell_price > 0) && $parent_node->get('parent_enroll_options')->value != 'parent') {
        // Course is for sale, and activity set is setup to allow purchase of this course.
        $title .= '<span class="price child-price"> - ' . uc_currency_format($node->sell_price) . '</span>';
      }

      if ($node->access('update')) {
        // Admin has access to update. Add a link to edit.
        // @todo fix for D9
        //$title .= ' ' . Link::fromTextAndUrl('[edit]', $node->getUrl());
      }

      if ($block) {
        $image = [
          '#theme' => 'image',
          '#uri' => 'misc/watchdog-error.png',
          '#title' => $blocker->getReason() ?? '',
        ];
        $image_out = render($image);

        $title = "$image_out $title<br/>{$blocker->getReason()}";
      }

      $child['#title'] = $title;


      if (course_relationships_is_parent($node)) {
        // The child is an activity set.
        $level++;
        $child = array_merge($child, $this->buildChildForm($node, $level));
      }
      else {
        // Child is a course. Build a checkbox.
        $child['#type'] = 'checkbox';
        $child['#return_value'] = $node->id();

        if (!$user->isAuthenticated()) {
          // Not logged in.
          $child['#disabled'] = TRUE;
        }

        $type = course_relationships_user_has_product($node->id());

        if ($type) {
          // User already purchased this course, so disable the checkbox.
          $child['#disabled'] = TRUE;
          $child['#value'] = $node->id();
          $record = $node->getEnrollment($user);
          $child['#description'] = $type == 'cart' ? '(' . l('in cart', 'cart') . ')' : "({$record->section_name->value})";
        }

        if ($block) {
          // User is blocked or can only purchase parent, so we don't want them selecting children.
          $child['#disabled'] = TRUE;
        }

        // Ensure user has view access to the child node.
        $child['#access'] = $node->access('view');
      }
      $children['activities'][$node->id()] = $child;
    }
    return $children;
  }


  /**
   * @param $values a parent
   */
  function course_relationships_recursive_add_to_cart(&$values) {
    global $user;
    static $ret = [];
    $delete_children = FALSE;
    $added = 0;

    $context = $values;
    if (!empty($values['parent'])) {
      $product = Course::load($values['parent']);
      \Drupal::moduleHandler()->alter('course_relationships_add_to_cart_context', $context);
      $ret[$values['parent']] = course_relationships_bulk_add_to_cart($product, $context);
      if ($product->get('parent_auto_enroll')->value == 'auto') {
        $delete_children = TRUE;
      }
      $added++;
      // add parent to cart, continue
    }
    if (is_array($values['activities'])) {
      if ($delete_children) {
        // Don't add any children, also delete them all.
        if (\Drupal::moduleHandler()->moduleExists('uc_cart')) {
          $count = db_delete('uc_cart_products')
            ->condition('nid', array_keys($values['activities']))
            ->condition('cart_id', $user->id())
            ->execute();
          if ($count) {
            drupal_set_message('Items have been removed from your cart because you are purchasing a set.', 'warning', FALSE);
          }
        }
      }
      else {
        // Process adding children to the cart.
        $activities = array_filter($values['activities']);
        foreach ($activities as $key => $item) {
          if (!is_array($item)) {
            if (!course_relationships_user_has_product($item) && $key == $item) {
              //add course to cart, end here
              $product = Course::load($item);
              \Drupal::moduleHandler()->alter('course_relationships_add_to_cart_context', $context);
              $ret[$item] = course_relationships_bulk_add_to_cart($product, $context);
              $added++;
            }
          }
          else {
            // this is an activity set , keep going
            $this->course_relationships_recursive_add_to_cart($item);
          }
        }
      }
    }

    return $ret;
  }

}
