<?php

namespace Drupal\course_relationships\Plugin\course\CourseObject;

/**
 * @file
 * Course object classes for course relationships.
 */

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\course\Entity\Course;
use Drupal\course\Entity\CourseObject;
use Drupal\course\Plugin\course\CourseObject\CourseObjectEntity;

/**
 * CourseObject that holds a Course.
 *
 * @CourseObject(
 *   id = "course",
 *   label = "Course",
 *   handlers = {
 *     "fulfillment" = "\Drupal\course\Entity\CourseObjectFulfillment"
 *   }
 * )
 */
class CourseObjectCourse extends CourseObjectEntity {

  /**
   * Set up defaults.
   */
  function defaultConfiguration() {
    $definition = parent::defaultConfiguration();
    $definition['course_outline'] = 'course';
    $definition['course_goto'] = 'view';
    $definition['private'] = 0;
    $definition['instanceBundle'] = 'course';
    return $definition;
  }

  function buildConfigurationForm($form, FormStateInterface $form_state) {
    $form += parent::buildConfigurationForm($form, $form_state);
    $defaults = $this->getConfiguration();

    // Course outline display handler.
    $outlines = [];
    $handlers = course_get_outline_handlers();
    foreach ($handlers as $key => $handler) {
      $outlines[$key] = $handler['label'];
    }

    if (empty($defaults['instance'])) {
      $form['course']['course_outline'] = [
        '#title' => t('Course outline display'),
        '#type' => 'select',
        '#options' => $outlines,
      ];
    }

    $form['course']['course_goto'] = [
      '#type' => 'select',
      '#title' => t('Course entry point'),
      '#options' => [
        'view' => t('View Course'),
        'take' => t('Take Course'),
      ],
      '#default_value' => $defaults['course_goto'],
      '#description' => t('Selecting "Take Course" will launch the user directly into taking the course, without viewing the course body.'),
    ];

    return $form;
  }

  /**
   * Validate the referenced course to make sure that it does not contain this
   * course.
   */
  function validateConfigurationForm(&$form, FormStateInterface $form_state) {
    if ($form_state->getValue('instance')) {
      $course = Course::load($form_state->getValue('instance'));
      foreach ($course->getObjects() as $courseObject) {
        if ($courseObject->getInstanceId() == $this->getCourse()->id()) {
          $form_state->setError($form['course']['instance'], t('Circular dependency detected. The specified course cannot contain the current course.'));
        }
      }

      if ($course->id() == $this->getCourse()->id()) {
        $form_state->setError($form['course']['instance'], t('Circular dependency detected. The specified course cannot be the current course.'));
      }
    }

    parent::validateConfigurationForm($form, $form_state);
  }

  /**
   * Create the new course with our node type and outline selection.
   */
  public function createInstance($node = NULL) {
    $course = Course::create(['type' => 'course']);
    $course->set('title', $this->getTitle());
    $course->set('outline', $this->getOption('course_outline'));
    $course->setOwnerId(\Drupal::currentUser()->id());
    $course->save();
    $this->setInstanceId($course->id());
  }

  /**
   * The take URL of the course is /takecourse.
   */
  function getTakeUrl() {
    if ($this->getOption('course_goto') == 'take') {
      return Url::fromRoute('course.take', ['course' => $this->getInstanceEntity()->id()]);
    }

    return Url::fromRoute('entity.course.canonical', ['course' => $this->getInstanceId()]);
  }

  public function getTakeType() {
    return 'redirect';
  }

  /**
   * {@inheritdoc}
   */
  function getEditUrl() {
    $url = Url::fromRoute('entity.course.edit_form', ['course' => $this->getInstanceId()]);
    return $url;
  }

  public function getInstanceEntityType(): string {
    return 'course';
  }

}
